//======================================================== IBJmsg.h
//
// Message Routines for IBJ-programs
// =================================
//
// Copyright (C) Janicke Consulting, Dunum, Germany, 1998-2001
// email: info@janicke.de
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// history:
//
// last change: 98-07-31  lj
// 2000-07-27 uj  extended for IBJodor
// 2001-09-03 lj  MsgUnquote()
//
//==================================================================

#ifndef  IBJMSG_INCLUDE
#define  IBJMSG_INCLUDE

#include <stdio.h>

#define  SYMOPT   "-"
#define  PATHSEP  '/'
#define  ALTPSEP  '\\'
#define  MAXFNLEN 256
#define  EOL      "\n"

#ifndef dP
  #define  dP(a)   char *ePGMn = #a; char ePGMs[256]
  #define  dQ(a)   char *ePGMn = #a
  #define  eG(a)   if(MsgCode<0) eX(a)
  #define  eX(a)   {sprintf(ePGMs,"%s.%s.%s",eMODn,ePGMn,#a); MsgCode=-1; goto eX_ ## a;}
#endif

#define  eMSG    return MsgCode=-1, vMsg("***"), vMsg
#define  nMSG    vMsg("***"), vMsg

#ifndef ALLOC
  #ifdef LOG_ALLOCATION
    #define ALLOC(a) (MsgPvd=malloc(a)); if(MsgPvd)memset(MsgPvd,0,(a)); \
            if (MsgFile){ fprintf(MsgFile,"ALLOC %08x(%d) in %s\n", \
            MsgPvd,(int)(a),ePGMn); fflush(MsgFile);}
    #define CALLOC(a) (MsgPvd=calloc((a),(b)));\
            if (MsgFile){ fprintf(MsgFile,"CALLOC %08x(%d,%d) in %s\n", \
            MsgPvd,(int)(a),(int)(b),ePGMn); fflush(MsgFile);}
    #define REALLOC(a,b) (MsgPvd=realloc((a),(b)));\
            if (MsgFile){ fprintf(MsgFile,"REALLOC %08x=>%08x(%d) in %s\n",\
            (int)(a),MsgPvd,(int)(b),ePGMn); fflush(MsgFile);}
    #define FREE(p) {if (MsgFile){ fprintf(MsgFile, "FREE %08x in %s\n",\
            (p),ePGMn); fflush(MsgFile);} free(p);}
  #else
    #define ALLOC(a)        calloc(1,(a))
    #define CALLOC(a,b)     calloc((a),(b))
    #define FREE(a)         free(a)
    #define REALLOC(a,b)    realloc((a),(b))
  #endif
#endif

extern char *IBJmsgVersion;
extern int MsgCode, MsgQuiet, MsgVerbose, MsgBufLen, MsgBreak;
extern int MsgTranslate, MsgDateYear, MsgDateMonth, MsgDateDay;
extern int MsgDateHour, MsgDateMinute, MsgDateSecond;
extern int MsgTranslate;
extern FILE *MsgFile;
extern void *MsgPvd;

void MsgLow(    /* convert to lower case        */
char *s )       /* string to be converted       */
  ;
char *MsgTok(   /* non-destructive tokenisation */
char *start,    /* begin of scanning            */
char **pnext,   /* start of next scan           */
char *sep,      /* separator characters         */
int *pl )       /* number of usable characters  */
  ;             /* RETURN: start of token       */
char **_MsgTokens(  // tokenize a string
char *t,            // string to be tokenized (changed by this routine!)
char *s )           // separator characters
;                   // RETURN: allocated array of strings, ends with NULL
int MsgDequote( //  remove quotation marks and escape sequences
char *s )       //  string to work on
;               //  RETURN: number of replacements
int MsgUnquote( //  remove quotation marks
char *s )       //  string to work on
;               //  RETURN: number of replacements

int vMsg(       /* print message on screen and into logfile     */
char *format,   /* printing format                              */
... )           /* parameter                                    */
  ;
double MsgDateVal(      /* converts date string into numerical value    */
char * ss )             /* string of the form yy-mm-dd'hh:mm:ss         */
  ;                     /* RETURN: days after 1899-12-30'00:00:00 +1.e6 */

char *MsgDateString(    /* converts date value into string      */
double d )              /* days after 1899-12-30'00:00:00 +1.e6 */
  ;                     /* RETURN: string yy-mm-dd'hh:mm:ss     */

double MsgDateSeconds(  /* converts time from t1 to t2 into seconds */
double t1,
double t2 )
  ;
double MsgDateIncrease(   // increase date t by n seconds
double t,
int n )
;
int MsgCheckPath(     // normalize path expression
char *s )             // path
;                     // RETURN: is absolute path

#endif  /*##################################################################*/
