//======================================================== IBJstt.c
//
// Generation of Input Files from DWD Statistics
// =============================================
//
// Copyright (C) Janicke Consulting, Dunum, Germany, 1999-2001
// email: info@janicke.de
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// last change: 2001-08-01 uj
//
//==================================================================

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <math.h>
#include <time.h>

#include "IBJstt.h"

char *SttVersion = "1.5";

#define  NKL    6
#define  NKLalt 7
#define  NWR    36  /* darf nicht verndert werden */
#define  NWRalt 38
#define  NWGneu  9
#define  NWG    15
#define  NWGalt 32
#define  STT_BUFLEN   4000

#ifdef MAIN
  static float Lm, Hm, Dw[NWR], Ua, Zrgh, Rw;
  static float Ew, Hs[3], Py[3], Pz[3], Qy[3], Qz[3];
  static int Nw = NWR, Nh = 3;
  static int eCode;
  static int MsgBufLen = 4000;
  static int MsgVerbose, MsgQuiet;

  #define  LM   Lm
  #define  HM   Hm
  #define  UA   Ua
  #define  Z0   Zrgh
  #define  DW(i)  Dw[i]
  #define  NW   Nw
  #define  EW   Ew
  #define  RW     Rw
  #define  NH     Nh
  #define  HS(i)  Hs[i]
  #define  PY(i)  Py[i]
  #define  PZ(i)  Pz[i]
  #define  QY(i)  Qy[i]
  #define  QZ(i)  Qz[i]
  #define  ALLOC(a)        calloc(1,(a))
  #define  FREE(a)         free(a)
  #define  dP(a)  char *ePGMpgm = #a; char ePGMs[80]
  #define  eG(a)  if (eCode<0) {sprintf(ePGMs, "%s-%s", ePGMpgm, #a); \
                  goto eX_##a;}
  #define  eX(a)  {sprintf(ePGMs,"%s-%s",ePGMpgm,#a);eCode=-1;goto eX_##a;}

  static int vMsg(  // print message on screen and into logfile
  char *format,     // printing format
  ... )             // parameter
  ;
#else
  #include "IBJtbl.h"
  #include "IBJmsg.h"
#endif

static char *eMODn = "IBJstt";

static float Stat[NKL][NWG][NWR];
static float StatAlt[NKLalt][NWGalt][NWRalt];
static int Nwg, StatFormat;
static char *FormatName[] = { "NEUES FORMAT", "ALTES FORMAT", "VDI FORMAT" };
static char *ClassName[] = { "I", "II", "III/1", "III/2", "IV", "V" };

static char Hdr[5][160];

static float WgTAL[9]  = {  1.0,  1.5,  2.0,  3.0,  4.5,  6.0,  7.5,  9.0,  12.0 };
static float WgVDI[15] = {  1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15 };
static float Wg[NWG];
static float Kl[NKL] = {  1.0,  2.0,  3.1,  3.2,  4.0,  5.0 };
static float *Ew0;
static float Ew1[NKL] = { 0.37, 0.32, 0.26, 0.18, 0.14, 0.12 };  /* glattes Gelnde */
static float Ew2[NKL] = { 0.52, 0.48, 0.31, 0.31, 0.31, 0.20 };  /* rauhes Gelnde  */
static float Ew3[NKL] = { 0.42, 0.37, 0.28, 0.22, 0.20, 0.09 };  /* TA-Luft         */
static float *Hm0;
static float Hm1[NKL] = {  250,  250,  800,  800, 1100, 1100 };  /* VDI 3782/1 */
static float Hm2[NKL] = { 1.e6, 1.e6, 1.e6, 1.e6, 1.e6, 1.e6 };  /* TA-Luft    */
static float Hm3[NKL] = {    0,    0,    0,  800, 1100, 1100 };  /* LASAT      */
static float Hm4[NKL] = {  150,  600,  800,  800, 1100, 1100 };  /* LASAT-TAL  */
static float *Lmo;
static float Lm1[NKL] = { 70, 250, 99999, -300, -150, -70 };     /* LASAT */
static float Lm2[NKL] = { 80, 2000, -280,  -86,  -42, -13 };     /* TA-Luft 50 */
static float Lm4[NKL] = { 90, 350, 99999, -300, -190, -80 };     /* LASAT-TAL  */


static float StatWeight = 1.0;
static int Mode = 1;
static int IwgRef = 2;
static int Geruch = 0;
static int TaLuft, Isotropic;

static char StatName[256], Pfad[256];

#ifdef  EW
static int Nsig = 3;
static float HsigNEW[3] = { 50, 100, 180 };
static struct { float F, f, G, g; } SigTabNEW[3][NKL] =
  {{{ 1.294, 0.718, 0.241, 0.662 },
    { 0.801, 0.754, 0.264, 0.774 },
    { 0.640, 0.784, 0.215, 0.885 },
    { 0.659, 0.807, 0.165, 0.996 },
    { 0.876, 0.823, 0.127, 1.108 },
    { 1.503, 0.833, 0.151, 1.219 }},
   {{ 0.253, 1.057, 0.717, 0.486 },
    { 0.411, 0.882, 0.487, 0.652 },
    { 0.504, 0.818, 0.265, 0.818 },
    { 0.466, 0.866, 0.137, 0.985 },
    { 0.324, 1.025, 0.070, 1.151 },
    { 0.170, 1.296, 0.051, 1.317 }},
   {{ 0.671,  0.903,  0.484,  0.500  },
    { 0.345,  0.903,  0.546,  0.557  },
    { 0.208,  0.903,  0.307,  0.734  },
    { 0.232,  0.903,  0.104,  0.997  },
    { 0.415,  0.903,  0.0330, 1.320  },
    { 0.671,  0.903,  0.0245, 1.500  }}};

static float HsigTAL[3] = { 50, 100, 150 };    
static struct { float F, f, G, g; } SigTabTAL[3][NKL] =
    {{{ 1.294, 0.718, 0.241, 0.662 },
    { 0.801, 0.754, 0.264, 0.774 },
    { 0.640, 0.784, 0.215, 0.885 },
    { 0.659, 0.807, 0.165, 0.996 },
    { 0.876, 0.823, 0.127, 1.108 },
    { 1.503, 0.833, 0.151, 1.219 }},
     {{ 0.253, 1.057, 0.717, 0.486 },
    { 0.411, 0.882, 0.487, 0.652 },
    { 0.504, 0.818, 0.265, 0.818 },
    { 0.466, 0.866, 0.137, 0.985 },
    { 0.324, 1.025, 0.070, 1.151 },
    { 0.170, 1.296, 0.051, 1.317 }},
     {{ 0.31,  0.71,  0.06,  0.71  },
    { 0.31,  0.71,  0.06,  0.71  },
    { 0.32,  0.78,  0.22,  0.78  },
    { 0.36,  0.86,  0.33,  0.86  },
    { 0.40,  0.91,  0.41,  0.91  },
    { 0.40,  0.91,  0.41,  0.91  }}};
#endif

//=============================================================== ReadStatAlt
static int ReadStatAlt( FILE *ein )
  {
  dP(ReadStatAlt);
  char buf[800], *pt;
  int ikl, iwg, iwr, ig;
  float summea, summeb, faktor, a;
  for (ikl=0; ikl<NKLalt; ikl++) {
    summea = 0;
    for (iwg=0; iwg<NWGalt; iwg++) {
      if (NULL == fgets( buf, 800, ein ))                       eX(3);
      pt = strtok( buf, " ,\t\n\r" );
      for (iwr=0; iwr<NWRalt; iwr++) {
        if (!pt)                                                eX(4);
        if (1 > sscanf( pt, "%f", &a ))                         eX(5);
        StatAlt[ikl][iwg][iwr] = a;
        summea += a;
        pt = strtok( NULL, " ,\t\n\r" );
        }
      }
    /* printf("In Klasse %d: Summe=%4.0f\n", ikl+1, summea); */
    }
  for (iwg=0; iwg<Nwg; iwg++)  Wg[iwg] = (TaLuft) ? WgTAL[iwg] : WgVDI[iwg];
  //
  // bertragen der Klasse UNBEKANNT
  //
  for (iwr=0; iwr<NWRalt; iwr++) {
    for (iwg=0; iwg<=4; iwg++)  StatAlt[0][iwg][iwr] += StatAlt[6][iwg][iwr];
    for (iwg=5; iwg<=6; iwg++)  StatAlt[1][iwg][iwr] += StatAlt[6][iwg][iwr];
    for (iwg=7; iwg<NWGalt; iwg++)
      StatAlt[2][iwg][iwr] += StatAlt[6][iwg][iwr];
    }
  //
  // Umverteilen der Kalmen und Schwachwindlagen
  //
  for (ikl=0; ikl<NKL; ikl++) {
    summea = 0;
    for (iwr=0; iwr<NWR; iwr++)  summea += StatAlt[ikl][IwgRef][iwr+1];
    for (iwg=1; iwg<IwgRef; iwg++) {
      summeb = (iwg == 1) ? StatAlt[ikl][0][0] : 0;
      for (iwr=0; iwr<NWR+1; iwr++) {
          summeb += StatAlt[ikl][iwg][iwr+1];
        }
      if (summea > 0)  faktor = summeb/summea;
      else {
        faktor = 0;
        summeb /= NWR;
        }
      for (iwr=0; iwr<NWR; iwr++)
        if (faktor)
          StatAlt[ikl][iwg][iwr+1] = faktor*StatAlt[ikl][IwgRef][iwr+1];
        else  StatAlt[ikl][iwg][iwr+1] = summeb;
      StatAlt[ikl][iwg][NWR+1] = 0;
      }
    StatAlt[ikl][0][0] = 0;
    }
  //
  // Neuverteilen der umlaufenden Winde + bertragen
  //
  for (ikl=0; ikl<NKL; ikl++)
    for (iwg=1; iwg<NWGalt; iwg++) {
      if (TaLuft) {
        if (iwg < 5)  ig = (iwg > 1) ? iwg - 2 : 0;
        else          ig = (iwg + 4)/3;
        }
      else  ig = (iwg-1)/2;
      if (ig >= Nwg)  ig = Nwg-1;
      summea = 0;
      for (iwr=0; iwr<NWR; iwr++)  summea += StatAlt[ikl][iwg][iwr+1];
      summeb = StatAlt[ikl][iwg][NWR+1];
      if (summea > 0)  faktor = (summea+summeb)/summea;
      else {
        faktor = 0;
        summeb /= NWR;
        }
      for (iwr=0; iwr<NWR; iwr++)
        if (faktor)  Stat[ikl][ig][iwr] += faktor*StatAlt[ikl][iwg][iwr+1];
        else  Stat[ikl][ig][iwr] += summeb;
      if (ikl==0 && iwg==4)  break;
      if (ikl==1 && iwg==6)  break;
      }
  return 1;
eX_3:  eX_4:  eX_5:
  vMsg("*** File unvollstndig! (%s)\n", ePGMs);
  return -1;
  }

/*================================================================== ReadStat
*/
int ReadStat( int seqnum )
  {
  dP(ReadStat);
  char buf[800], *pt, *pc, name[256], fname[256];
  int i, ikl, iwg, iwr, l, nn, n;
  FILE *ein;
  float a, summe;
  StatWeight = 1.0;
  pc = StatName;
  for (i=1; i<seqnum; i++) {
    pc = strchr( pc, ';' );  if (!pc)   return 0;
    pc++;
    }
  strcpy( name, pc );
  pc = strchr( name, ';' );
  if (pc)  *pc = '\0';
  if (!*name)  return 0;
  pc = strchr(name,'(');
  if (pc) {
    sscanf(pc+1, "%f", &StatWeight);
    if (StatWeight <= 0)                                        eX(7);
    *pc = '\0';
    }
  if (*name == '~') {
    strcpy(fname, Pfad);
    strcat(fname, name+1);
  }
  else {
      // if (strchr(name,'/'))  *fname = '\0';
      // else  strcpy(fname, Pfad);
      // strcat(fname, name);
      strcpy(fname, name);
  }
  ein = fopen( fname, "rb" );
  if (ein == NULL)                                              eX(1);
  nn = 0;
  for (i=0; i<5; i++) {
    if (NULL == fgets( Hdr[i], 120, ein ))                      eX(2);
  nn++;
    for (l=strlen(Hdr[i])-1; l>=0; l--)
      if ((unsigned char)Hdr[i][l] < ' ')  Hdr[i][l] = ' ';
    Hdr[i][78] = 0;
    printf( "%s\n", Hdr[i] );
    if (i == 2) {
      if (strstr(Hdr[i], "ALTES FORMAT")) {
        StatFormat = 2;
        Nwg = (TaLuft) ? 9 : 15;
        }
      else {
        StatFormat = 0;
        Nwg = 9;
        }
      }
    }
  for (ikl=0; ikl<NKL; ikl++)
    for (iwg=0; iwg<Nwg; iwg++)
      for (iwr=0; iwr<NWR; iwr++)  Stat[ikl][iwg][iwr] = 0;
  if (StatFormat == 2) {
    n = ReadStatAlt(ein);                                        eG(10);
  if (n < 1)  return 0;
    if (TaLuft)  StatFormat = 0;
    summe = 0;
    for (ikl=0; ikl<NKL; ikl++) {
      for (iwg=0; iwg<Nwg; iwg++)
        for (iwr=0; iwr<NWR; iwr++)
          summe += Stat[ikl][iwg][iwr];
      /* printf("Bis Klasse %d:  Summe=%5.0f\n", ikl+1, summe); */
      }
    fclose(ein);
    vMsg("Summe Statistik %d: %5.0f\n", seqnum, summe);
    return 1;
    }
  summe = 0;
  for (ikl=0; ikl<NKL; ikl++)
    for (iwg=0; iwg<Nwg; iwg++) {
      if (NULL == fgets( buf, 800, ein ))                       eX(3);
    nn++;
      pt = strtok( buf, " ,\t\n\r" );
      for (iwr=0; iwr<NWR; iwr++) {
        if (!pt)                                                eX(4);
        if (1 > sscanf( pt, "%f", &a ))                         eX(5);
        Stat[ikl][iwg][iwr] = a;
        summe += a;
        pt = strtok( NULL, " ,\t\n\r" );
        }
      }
  fclose(ein);
  vMsg("Summe Statistik %d: %5.0f\n", seqnum, summe);
  for (iwg=0; iwg<Nwg; iwg++)  Wg[iwg] = WgTAL[iwg];
  return 1;
eX_1:
  vMsg("*** File %s nicht lesbar! (%s)\n", fname, ePGMs);
  return -1;
eX_2:  eX_3:
  vMsg("*** File %s unvollstndig (line %d)! (%s)\n", fname, nn, ePGMs);
  return -1;
eX_4:  eX_5:
  vMsg("*** File %s unvollstndig (line %d, %d/%d/%d)! (%s)\n",
          fname, nn, ikl, iwg, iwr, ePGMs);
  return -1;
eX_7:
  vMsg("*** Skalierung der Wetterstatistik nicht positiv! (%s)\n", ePGMs);
  return -1;
eX_10:
  vMsg("*** File %s (ALTES FORMAT) nicht lesbar! (%s)\n", fname, ePGMs);
  return -1;
  }

//================================================================== PrintStat
static int PrintStat( FILE *p ) {
  int i, iwr, ikl, iwg;
  char *pc;
  if (!p)  return 0;
  for (i=0; i<5; i++) {
    if (i == 2) {
      pc = strstr(Hdr[i], "ALTES FORMAT");
      if (pc) {
        *pc = 0;
        strcat(Hdr[i], FormatName[StatFormat]);
        strcat(Hdr[i], ")");
        }
      }
    fprintf(p, "%s\n", Hdr[i]);
    }
  for (ikl=0; ikl<NKL; ikl++)
    for (iwg=0; iwg<Nwg; iwg++) {
      for (iwr=0; iwr<NWR; iwr++)  fprintf(p, " %4.0f", Stat[ikl][iwg][iwr]);
      fprintf(p, "\n");
      }
  return 0;
  }

/*=========================================================== SttWriteCmd
*/
int SttWriteCmd( FILE *p, int seqnum, int (*pgm)(void) )
{
  dP(SttWriteCmd);
  int n, iwr, iwg, ikl;
  float summe, summekl[NKL];
  if (p)  fprintf(p, "*Parameter \n");
#ifdef  LM
  if (p)  fprintf(p, "Z0 %6.4f \n", Z0);
#endif
#ifdef  RW
  if (seqnum > 1) {
    RW = 0;
    if (p)  fprintf(p, "RW %10.3e \n", RW);
  }
#endif
#ifdef  KL
  strcpy(KL, "*");
#endif
  for (ikl=0; ikl<NKL; ikl++) {
    summekl[ikl] = 0;
    HM = Hm0[ikl];
    if (Mode & 0x08) {
#ifdef  LM
      // HM = 0;                               //-00-01-11
      LM = Lmo[ikl];
      if (ikl != 2)  LM *= sqrt(Z0/1.5);
#endif  // LM
    }
    else {
#ifdef  EW
      EW = Ew0[ikl];
      NH = Nsig;
      if (TaLuft) {
        for (n=0; n<Nsig; n++) {
          HS(n) = HsigTAL[n];
          PY(n) = SigTabTAL[n][ikl].F;
          QY(n) = SigTabTAL[n][ikl].f;
          PZ(n) = SigTabTAL[n][ikl].G;
          QZ(n) = SigTabTAL[n][ikl].g;
        }
      }
      else {
        for (n=0; n<Nsig; n++) {
          HS(n) = HsigNEW[n];
          PY(n) = SigTabNEW[n][ikl].F;
          QY(n) = SigTabNEW[n][ikl].f;
          PZ(n) = SigTabNEW[n][ikl].G;
          QZ(n) = SigTabNEW[n][ikl].g;
        }
      }        
#endif  // EW
    }
    if (p) {
      fprintf(p, "*Parameter ' Klasse %3.1f \n", Kl[ikl]);
      if (Mode & 0x08) {
#ifdef  LM
        fprintf(p, "Hm %4.0f \n", HM);
        fprintf(p, "Lm %7.1f \n", LM);
#endif  // LM
      }
      else {
#ifdef  EW
        fprintf(p, "Hm %4.0f \n", HM);
        fprintf(p, "Ew %4.2f \n", EW);
        fprintf(p, "Hs");
        for (n=0; n<NH; n++)  fprintf(p, " %5.0f", HS(n));
        fprintf(p, " \n");
        fprintf(p, "Py" );
        for (n=0; n<NH; n++)  fprintf(p, " %5.3f", PY(n));
        fprintf(p, " \n");
        fprintf(p, "Qy");
        for (n=0; n<NH; n++)  fprintf(p, " %5.3f", QY(n));
        fprintf(p, " \n");
        fprintf(p, "Pz");
        for (n=0; n<NH; n++)  fprintf(p, " %5.3f", PZ(n));
        fprintf(p, " \n");
        fprintf(p, "Qz" );
        for (n=0; n<NH; n++)  fprintf(p, " %5.3f", QZ(n));
        fprintf(p, " \n");
#endif  // EW
      }
    }
    for (iwg=0; iwg<Nwg; iwg++) {
      UA = Wg[iwg];
      summe = 0;
      for (iwr=0; iwr<NWR; iwr++)  summe += Stat[ikl][iwg][iwr];
      summekl[ikl] += summe;
      if (summe <= 2)  continue;
      for (iwr=0; iwr<NWR; iwr++)
        if (Isotropic)  DW(iwr) = summe*StatWeight/NWR;
        else            DW(iwr) = Stat[ikl][iwg][iwr]*StatWeight;
      if (p) {
        fprintf(p, "*Parameter run \n");
        fprintf(p, "Ua %4.1f \n", UA);
        fprintf(p, "Dw");
        for (iwr=0; iwr<NWR; iwr++)  fprintf(p, " %5.1f", DW(iwr));
        fprintf(p, " \n");
        fprintf(p, "* \n");
      }
      if (pgm) {
        if (LM != 0) vMsg("Ua=%1.2f, Lm=%1.1f, Hm=%1.1f\n", UA, LM, HM);
        else vMsg("KL=%5s, HM=%10.0f UA=%5.2f, EW=%1.2f, WEIGHT=%6.0f\n", ClassName[ikl], HM, UA, EW, summe);
        n = pgm();  if (n < 0)              eX(1);
      }
    }  // for iwg
  }  // for ikl
  if (pgm) {
    summe = 0;
    for (summe=0,ikl=0; ikl<NKL; ikl++) {
      vMsg("KL=%5s, WEIGHT=%6.0f\n", ClassName[ikl], summekl[ikl]);
      summe += summekl[ikl];
    }
    vMsg("TOTAL WEIGHT    =%6.0f\n", summe);
  }
  return 0;
eX_1:
  vMsg("*** Fehler bei der Abarbeitung der Statistik! (%s)\n", ePGMs);
  return -1;
}

//================================================================= SttWork
int SttWork(        // Wetterstatistik abarbeiten
  char *path,       // Arbeitsverzeichnis
  char *options,    // Optionen
  char *statname,   // File-Name(n) der Wetterstatistik(en)
  char *cmdname,    // Name des CMD-Files oder NULL
  int iwgref,       // Referenzwert fr Umverteilung der Kalmen (kn)
  float z0,         // Rauhigkeitslnge (m)
  int (*pgm)(void)) // Auszufhrendes Programm oder NULL
{
  dP(SttWork);
  FILE *cmd;
  int i, n, print_stat = 0;
  char fn[256], s[80], *pc;
  if (NWR != NW)                eX(0);
  strcpy(Pfad, path);
  strcpy(StatName, statname);
  for (n=0; n<strlen(StatName); n++)
    if (StatName[n] == '\\')  StatName[n] = '/';
  Mode = 0x01;
  IwgRef = 2;
  Geruch = 0;
  TaLuft = 0;
  Ew0 = Ew1;
  Hm0 = Hm1;
  Lmo = Lm1;
  Isotropic = 0;
  if (iwgref > 0)  IwgRef = iwgref;
  if (strchr(options, 'G')) {
    Mode = 0x08;
    if (strchr(options, 'O')) {
      Geruch = 1;
      Mode |= 0x10;
    }
    if      (strchr(options, 'V'))      Hm0 = Hm3;              //-00-05-04
    if      (strstr(options, "T50"))  { Hm0 = Hm4;  Lmo = Lm2; }
    else if (strstr(options, "T100")) { Hm0 = Hm4;  Lmo = Lm4; }
  }
  else if (strchr(options, 'T')) {
    TaLuft = 1;
    Mode = 0x02;
    IwgRef = 3;
    Hm0 = Hm2;
    Ew0 = Ew3;
    vMsg("RECHNUNG NACH TA Luft 1986\n");
  }
  if (strchr(options, 'g')) {
    Ew0 = Ew2;
  }
  if (strchr(options, 'P'))  print_stat = 1;
  if (strchr(options, 'I'))  Isotropic = 1;
  for (i=0; i<NKL; i++) {
    sprintf(s, "L[%d]=", i+1);
    pc = strstr(options, s);
    if (pc) {
      sscanf(pc+5, "%f", Lmo+i);
      vMsg("using Lmo[%d]=%1.1f\n", i+1, Lmo[i]);
printf("using Lmo[%d]=%1.1f\n", i+1, Lmo[i]);
    }
  }
#ifdef  Z0
  Z0 = z0;
#endif

  if ((cmdname) && (*cmdname)) {
    if (!strcmp(cmdname, "CON"))  cmd = stdout;
    else {
      strcpy(fn, path);
      strcat(fn, cmdname);
      cmd = fopen(fn, "wb");  if (!cmd) eX(1);
    }
  }
  else  cmd = NULL;
  n = ReadStat(1);                    eG(2);
  if (n > 0) {
    if (print_stat)  PrintStat(stdout);
    SttWriteCmd(cmd, 1, pgm);         eG(3);
  }
  n = ReadStat(2);                    eG(4);
  if (n > 0) {
    if (print_stat)  PrintStat(stdout);
    SttWriteCmd(cmd, 2, pgm);         eG(5);
  }
  if ((cmd) && (cmd!=stdout))  fclose(cmd);

  return 0;
eX_0:
  vMsg("*** Anzahl der Sektoren ungleich %d! (%s)\n", NWR, ePGMs);
  return -1;
eX_1:
  vMsg("*** File %s kann nicht angelegt werden! (%s)\n", fn, ePGMs);
  return -1;
eX_2: eX_4:
  vMsg("*** Wetterstatistik kann nicht eingelesen werden! (%s)\n", ePGMs);
  return -1;
eX_3: eX_5:
  if (cmd)
    vMsg("*** File %s kann nicht beschrieben werden! (%s)\n", fn, ePGMs);
  else
    vMsg("*** Fehler beim Abarbeiten der Wetterstatistik! (%s)\n", ePGMs);
  return -1;
}


#ifdef MAIN //#################################################################

static char Buf[STT_BUFLEN];

static int vMsg(  // print message on screen and into logfile
char *format,     // printing format
... )             // parameter
{
  va_list argptr;
  static char *s, *pc;
  static char DSTcode[] = { 0x84,0x94,0x81,0x8E,0x99,0x9A,0xE6,0xE1,0xFD,0xFC,0x00 };
  static char SRCcode[] = { 0xE4,0xF6,0xFC,0xC4,0xD6,0xDC,0xB5,0xDF,0xB2,0xB3,0x00 };
  int i, l;
  if (!format) {
    if (s)  FREE(s);
    s = NULL;
    return 0;
  }
  if (!s) {
    s = ALLOC(MsgBufLen);
    if (!s) {
      printf("*** can't allocate message buffer (%d bytes)!\n", MsgBufLen);
      exit(1);
    }
  }
  va_start(argptr, format);
  vsprintf(s, format, argptr);
  if (strlen(s) > MsgBufLen-1) {
    printf("*** message buffer overflow!\n");
    exit(1);
  }
  va_end(argptr);
  l = strlen(s);
  for (i=0; i<l; i++) {
    pc = strchr(SRCcode, s[i]);
    if (pc)  s[i] = DSTcode[pc-SRCcode];
  }
  printf("%s", s);
  return eCode;
}

//====================================================================== main
static void Help( void )
  {
  MsgQuiet = 0;
  MsgVerbose = 1;
  vMsg("\nIBJstat %s: Umwandlung Wetter-Statistik in CMD-File\n", SttVersion);
  vMsg("Copyright Janicke Consulting, Dunum, Germany, 1999-2001\n");
  vMsg("Compiled at %s  %s\n\n", __DATE__, __TIME__);
  vMsg("Aufruf:\n");
  vMsg("IBJstat <Pfad> [-G] [-g] [-K<KnRef>] [-p<Cmd>] [-T] [-V]\n"
     "       -w<Statistik>[,<Statistik>] [-z<Z0>]\n");
  vMsg("Pfad:       Name des Arbeitsverzeichnisses\n");
  vMsg("Statistik:  File-Name(Faktor) der DWD-Statistik trocken [na]\n");
  vMsg("-G          Geruchsausbreitung rechnen (IBJodor)\n");
  vMsg("-g          Alternative Windprofil-Exponenten (VDI, groe Rauhigkeit)\n");
  vMsg("Cmd:        Name fr den erzeugten CMD-File\n");
  vMsg("-T          Statistik nach TAL umwandeln, KnRef=3\n");
  vMsg("-V          Mischungsschichthhe nach VDI\n");
  vMsg("KnRef:      Referenzwert fr Umverteilung der Kalmen (=2 kn)\n");
  vMsg("Z0:         Rauhigkeitslnge\n");
  vMsg("\noder Aufruf: IBJstat [<Pfad>] @<Argumentfile>\n");
  exit(0);
}

void MyExit( void )
  { if (eCode)  vMsg("*** Programm abgebrochen\n"); }

int main( int argc, char *argv[] )
{
  dP(IBJstat);
  int n, l;
  int iwgref=2;
  char *pc, aname[256], options[32], path[256], statname[256], cmdname[256];
  float z0;
  FILE *argf;
  atexit(MyExit);
  if (argc < 2)  Help();
  argf = NULL;
  *options = 0;
  *path = 0;
  *statname = 0;
  strcpy(cmdname, "windrose.txt");
  z0 = 0.5;
  for (n=1; n<argc; ) {
    if (argf) {
      if (!fgets(Buf, STT_BUFLEN, argf)) {
        fclose(argf);
        argf = NULL;
        n++;
        continue;
      }
      pc = strchr(Buf, '#');
      if (pc)  *pc = '\0';
      pc = Buf;
      for (l=strlen(Buf)-1; l>=0 && Buf[l]<=' '; l--)  Buf[l] = '\0';
      if (!*pc)  continue;
    }
    else {
      pc = argv[n];
      if (*pc == '@') {
        strcpy(aname, Pfad);
        if (pc[1]) {
          strcat(aname, pc+1);
          pc = strrchr(aname, '/');
          if (!pc)  pc = aname;
          if (!strchr(pc, '.'))  strcat(aname, ".arg");
        }
        else  strcat(aname, "IBJstat.arg");
        argf = fopen(aname, "rb");  if (!argf)                        eX(1);
        continue;
      }
      n++;
    }
    if (*pc == '-')
    switch (pc[1]) {
      case 'G': strcat(options, "G");
                break;
      case 'g': strcat(options, "g");
                break;
      case 'h': Help();
                break;
      case 'I': strcat(options, "I");
                break;
      case 'K': sscanf(pc+2, "%d", &iwgref);
                break;
      case 'p': sscanf(pc+2, "%s", cmdname);
                break;
      case 'P': strcat(options, "P");
                break;
      case 'T': strcat(options, pc+1);
                break;
      case 'V': strcat(options, "V");         //-99-06-14
                break;
      case 'w': sscanf(pc+2, "%s", statname);
                break;
      case 'z': sscanf(pc+2, "%f", &z0);
                break;
      default:  printf("Unbekannte Option %s\n", pc );
                break;
    }
    else {
      if (*pc == '#')  continue;
      if (*pc == ' ')  continue;
      strcpy(path, pc);
      for (pc=path; (*pc); pc++)  if (*pc == '\\')  *pc = '/';
      if ((pc[-1]!=':') && (pc[-1]!='/')) {
        pc[0] = '/';
        pc[+1] = 0;
      }
    }
  }
  if (*statname == 0)  Help();
  vMsg("\nIBJstat %s: Umwandlung Wetter-Statistik in CMD-File\n", SttVersion);
  vMsg("Copyright Janicke Consulting, Dunum, Germany, 1999-2001\n");
  vMsg("Compiled at %s  %s\n", __DATE__, __TIME__);
  SttWork(path, options, statname, cmdname, iwgref, z0, NULL);    eG(2);
  vMsg("CMD-File %s ausgeschrieben.\n", cmdname);
  return 0;
eX_1:
    vMsg("*** Argumentfile %s nicht lesbar! (%s)\n", aname, ePGMs);
    return 1;
eX_2:
  vMsg("*** CMD-File kann nicht erstellt werden! (%s)\n", ePGMs);
  return 2;
}
#endif //####################################################################


// history:
// 99-01-29  lj  1.0
// 99-06-14  lj  1.1  mixing height modified (option 'V')
// 99-09-06  lj  1.2  option for modifying Lmo[]
// 00-01-11  lj  1.3  (hm=0 if Lm used)
// 00-05-04  lj  1.4  option Ltal for Lm4 and Hm4
// 01-05-10  uj  1.5  used by IBJplume, released as public


