// Copyright (C) Janicke Consulting, Dunum, Germany, 1998-2001
// email: info@janicke.de

// This is the header IBJtab.h to be used by the program "IBJplume".
// It defines all parameters whose values can be given by the user.
// It has to be included by all modules that use these values.

// Each parameter is defined in a record of the struct PrmTab.
// The name of the record is the same as the name of the input
// parameter (small letters). The record contains the name, a
// short description, input and output format and the default
// value as a string. For arrays also the number of elements or
// a reference to a parameter is given, that represents this
// number. For details see the header IBJprm.h.

#ifndef  IBJPRM_INCLUDE
    #include "IBJprm.h"  // The typedef of PRMTABREC is needed
#endif

#ifndef  IBJPRM_SOURCE
    extern  // Only IBJPRM saves the data of this structure
#endif
struct { PRMTABREC  // The names of the input parameters:
 _c, _d, mc, me, nh, np, nq, nw, op, ti, zp, _a, xp, yp, _p, nu, dw, re, kl, ha,
 ua, l0, a0, rh, vd, vs, ar, ur, ew, hm, hs, py, qy, pz, qz, _e, _t, _q, xq, yq,
 hq, eq, aq, bq, rq, uf, wf, ni, li, _g, nx, ny, x0, y0, dd, _s, qt, rv, fa, fo,
 fi, id, mo, se, _l, nd, _w, __, lm, z0, am, dm, n1, n2, ms, fz, hw, end_of_table; } PrmTab
#ifdef  IBJPRM_SOURCE
//      The initial values are defined in IBJPRM
 = {
 { "*c",  "%s", " \"%s\"",       "",  "0", 255,  0, 0x005, 0, "Werte auf dem Gitter oder den Aufpunkten Lschen" },
 { "*d",  "%s", " \"%s\"",       "",  "0", 255,  0, 0x005, 0, "Dimensionierung von Feldern und Rechengebiet" },
 { "mc",  "%d",   " %10d",   "4000",  "0",   4,  0, 0x002, 0, "Maximale Anzahl von Zeichen pro Zeile" },
 { "me",  "%s", " \"%s\"",      "g",  "0",  10,  0, 0x005, 0, "Mengeneinheit" },
 { "nh",  "%d",   " %10d",      "1",  "0",   4,  0, 0x002, 0, "Anzahl der Definitions-Hhen fr die Ausbreitungsparameter" },
 { "np",  "%d",   " %10d",      "0",  "0",   4,  0, 0x002, 0, "Anzahl der Aufpunkte" },
 { "nq",  "%d",   " %10d",      "1",  "0",   4,  0, 0x002, 0, "Anzahl der Quellen" },
 { "nw",  "%d",   " %10d",     "36",  "0",   4,  0, 0x002, 0, "Anzahl der Windrichtungen" },
 { "op",  "%s", " \"%s\"",       "",  "0",  10,  0, 0x005, 0, "Option (taluft/berljand)" },
 { "ti",  "%s", " \"%s\"",   "TEST",  "0",  10,  0, 0x005, 0, "Titel (idnt)" },
 { "zp",  "%f", " %10.1f",    "1.5",  "0",   4,  0, 0x003, 0, "Z-Koordinate der Aufpunkte (m)" },
 { "*a",  "%s", " \"%s\"",       "",  "0", 255,  0, 0x005, 0, "Koordinaten von Aufpunkten" },
 { "xp",  "%f", " %10.1f",      "0", "np",   4, -1, 0x103, 0, "X-Koordinate eines Aufpunktes (m)" },
 { "yp",  "%f", " %10.1f",      "0", "np",   4, -1, 0x103, 0, "Y-Koordinate eines Aufpunktes (m)" },
 { "*p",  "%s", " \"%s\"",       "",  "0", 255,  0, 0x005, 0, "Parameter der Ausbreitungsrechnung" },
 { "nu",  "%d",   " %10d",      "5",  "0",   4,  0, 0x002, 0, "Anzahl der Untersektoren fr eine Windrichtung" },
 { "dw",  "%f", " %10.0f",      "1", "nw",   4, -1, 0x103, 0, "Dauer der Windrichtung (h)" },
 { "re",  "%f", " %10.0f",     "10",  "0",   4,  0, 0x003, 0, "Richtung des ersten Sektors" },
 { "kl",  "%s", " \"%s\"",  "III/1",  "0",  15,  0, 0x005, 0, "Stabilittsklasse (Klug/Manier)" },
 { "ha",  "%f", " %10.0f",   "10.0",  "0",   4,  0, 0x003, 0, "Hhe des Anemometers ber Grund (m)" },
 { "ua",  "%f", " %10.2f",    "3.0",  "0",   4,  0, 0x003, 0, "Windgeschwindigkeit in Anemometerhhe (m/s)" },
 { "l0",  "%f", " %10.0f", "1000.0",  "0",   4,  0, 0x003, 0, "Charakt. Lnge bei Flchenquellen (m)" },
 { "a0",  "%d",   " %10d",     "50",  "0",   4,  0, 0x002, 0, "Maximale Anzahl von Teilquellen" },
 { "rh",  "%d",   " %10d",      "0",  "0",   4,  0, 0x002, 0, "Bodenrauhigkeit (0: mittel, 1: hoch) fr ew" },
 { "vd",  "%f", " %10.4f", "0.0000",  "0",   4,  0, 0x003, 0, "Depositionsgeschwindigkeit (m/s)" },
 { "vs",  "%f", " %10.4f", "0.0000",  "0",   4,  0, 0x003, 0, "Sedimentationsgeschwindigkeit (m/s)" },
 { "ar",  "%f", " %10.2e",    "0.0",  "0",   4,  0, 0x003, 0, "Auswaschrate (1/s)" },
 { "ur",  "%f", " %10.2e",    "0.0",  "0",   4,  0, 0x003, 0, "Umwandlungsrate (1/s)" },
 { "ew",  "%f", " %10.2f",   "0.31",  "0",   4,  0, 0x003, 0, "Exponent des Windprofils" },
 { "hm",  "%f", " %10.1f",  "800.0",  "0",   4,  0, 0x003, 0, "Hhe der Mischungsschicht (m)" },
 { "hs",  "%f", " %10.1f",  "100.0", "nh",   4, -1, 0x103, 0, "Definitions-Hhe(n) fr die Ausbreitungsparameter (m)" },
 { "py",  "%f", " %10.3f",  "0.504", "nh",   4, -1, 0x103, 0, "Ausbreitungsparameter: Faktor von Sigma-Y" },
 { "qy",  "%f", " %10.3f",  "0.818", "nh",   4, -1, 0x103, 0, "Ausbreitungsparameter: Exponent von Sigma-Y" },
 { "pz",  "%f", " %10.3f",  "0.265", "nh",   4, -1, 0x103, 0, "Ausbreitungsparameter: Faktor von Sigma-Z" },
 { "qz",  "%f", " %10.3f",  "0.818", "nh",   4, -1, 0x103, 0, "Ausbreitungsparameter: Exponent von Sigma-Z" },
 { "*e",  "%s", " \"%s\"",       "",  "0",  10,  0, 0x005, 0, "Ende der Parameterdatei" },
 { "*t",  "%s", " \"%s\"",       "",  "0",  10,  0, 0x005, 0, "Auflisten der verwendeten Eingabe-Parameter" },
 { "*q",  "%s", " \"%s\"",       "",  "0", 255,  0, 0x005, 0, "Quellen: Lage, berhhung und Quellstrke" },
 { "xq",  "%f", " %10.0f",      "0", "nq",   4, -1, 0x103, 0, "X-Koordinate des Quellmittelpunktes (m)" },
 { "yq",  "%f", " %10.0f",      "0", "nq",   4, -1, 0x103, 0, "Y-Koordinate des Quellmittelpunktes (m)" },
 { "hq",  "%f", " %10.1f",    "100", "nq",   4, -1, 0x103, 0, "Bauhhe der Quelle (m)" },
 { "eq",  "%f", " %10.2e",      "1", "nq",   4, -1, 0x103, 0, "Quellstrke (ME/s)" },
 { "aq",  "%f", " %10.1f",      "0", "nq",   4, -1, 0x103, 0, "Ausdehung der Quelle in x-Richtung vor der Drehung (m)" },
 { "bq",  "%f", " %10.1f",      "0", "nq",   4, -1, 0x103, 0, "Ausdehung der Quelle in y-Richtung vor der Drehung (m)" },
 { "rq",  "%f", " %10.0f",      "0", "nq",   4, -1, 0x103, 0, "Drehwinkel gegen den Uhrzeigersinn (Grad)" },
 { "uf",  "%f", " %10.1f",      "0", "nq",   4, -1, 0x103, 0, "Endberhhung der Abgasfahne (m)" },
 { "wf",  "%f", " %10.1f",      "0", "nq",   4, -1, 0x103, 0, "Weite des Fahnenanstiegs (m)" },
 { "ni",  "%d",   " %10d",    "100", "nq",   4, -1, 0x102, 0, "Anzahl der Intervalle in x-Richtung fr die Fahnenfunktionen" },
 { "li",  "%f", " %10.0f",     "30", "nq",   4, -1, 0x103, 0, "Lnge eines Intervalles (m)" },
 { "*g",  "%s", " \"%s\"",       "",  "0", 255,  0, 0x005, 0, "Gitter von Aufpunkten definieren" },
 { "nx",  "%d",   " %10d",      "0",  "0",   4,  0, 0x002, 0, "Anzahl der Punkte in x-Richtung" },
 { "ny",  "%d",   " %10d",      "0",  "0",   4,  0, 0x002, 0, "Anzahl der Punkte in y-Richtung" },
 { "x0",  "%f", " %10.0f",      "0",  "0",   4,  0, 0x003, 0, "X-Koordinate des linken Randes (m)" },
 { "y0",  "%f", " %10.0f",      "0",  "0",   4,  0, 0x003, 0, "Y-Koordinate des unteren Randes (m)" },
 { "dd",  "%f", " %10.0f",    "100",  "0",   4,  0, 0x003, 0, "Maschenweite (m)" },
 { "*s",  "%s", " \"%s\"",       "",  "0", 255,  0, 0x005, 0, "Speichern con|dry|wet|trn|qtl" },
 { "qt",  "%f", " %10.2e",   "0.95",  "0",   4,  0, 0x003, 0, "Quantil (Option qtl)" },
 { "rv",  "%f", " %10.2e",      "0",  "0",   4,  0, 0x003, 0, "Referenzwert (refv)" },
 { "fa",  "%f", " %10.2e",      "1",  "0",   4,  0, 0x003, 0, "Faktor" },
 { "fo",  "%s", " \"%s\"", "%12.2e",  "0",  10,  0, 0x005, 0, "Format" },
 { "fi",  "%s", " \"%s\"",       "",  "0", 255,  0, 0x005, 0, "File-Name" },
 { "id",  "%s", " \"%s\"",     """",  "0",  50,  0, 0x005, 0, "Identifikation (mark)" },
 { "mo",  "%s", " \"%s\"",   "text",  "0",  10,  0, 0x005, 0, "Modus der Datendarstellung (text|binary)" },
 { "se",  "%s", " \"%s\"",  "j-:i+",  "0",  10,  0, 0x005, 0, "Sequenz" },
 { "*l",  "%s", " \"%s\"",       "",  "0", 255,  0, 0x005, 0, "Fahnenwerte auflisten plm|pnt" },
 { "nd",  "%d",   " %10d",      "0",  "0",   4,  0, 0x002, 0, "Anzahl der beim Auflisten auszulassenden Punkte" },
 { "*w",  "%s", " \"%s\"",       "",  "0", 255,  0, 0x005, 0, "<Dateiname>  Wetterstatistik abarbeiten" },
 { "*_",  "%s", " \"%s\"",   "INTR",  "0",  10,  0, 0x005, 0, "Interne Parameter" },
 { "lm",  "%f", " %10.1f",      "0",  "0",   4,  0, 0x003, 0, "Lm" },
 { "z0",  "%f", " %10.1f",      "0",  "0",   4,  0, 0x003, 0, "Z0" },
 { "am",  "%f", " %10.0f",    "100",  "0",   4,  0, 0x003, 0, "Begrenzung des Argumentes der Exp-Funktion" },
 { "dm",  "%f", " %10.1e", "1.e-10",  "0",   4,  0, 0x003, 0, "Minimum fr die Determinante" },
 { "n1",  "%d",   " %10d",      "2",  "0",   4,  0, 0x002, 0, "Anzahl der Sttzpunkte fr 1. Gau-Quadratur" },
 { "n2",  "%d",   " %10d",      "4",  "0",   4,  0, 0x002, 0, "Anzahl der Sttzpunkte fr 2. Gau-Quadratur" },
 { "ms",  "%d",   " %10d",     "10",  "0",   4,  0, 0x002, 0, "Maximale Anzahl von Spiegelungen" },
 { "fz",  "%f", " %10.1f",    "1.5",  "0",   4,  0, 0x003, 0, "Begrenzung von Sigma-z (= Fz*Hm)" },
 { "hw",  "%f", " %10.1f",  "200.0",  "0",   4,  0, 0x003, 0, "Hhe, bis zu der das Windprofil gilt (m)" },
 { "**", NULL, NULL, NULL, NULL, 0, 0, 0, NULL } }
#endif
;
//  Macros for easier reference:

#define  MC  PrmTab.mc.u.i
#define  ME  (char*)PrmTab.me.u.p
#define  NH  PrmTab.nh.u.i
#define  NP  PrmTab.np.u.i
#define  NQ  PrmTab.nq.u.i
#define  NW  PrmTab.nw.u.i
#define  OP  (char*)PrmTab.op.u.p
#define  TI  (char*)PrmTab.ti.u.p
#define  ZP  PrmTab.zp.u.f
#define XP(i) ((float*)PrmTab.xp.u.p)[i]
#define YP(i) ((float*)PrmTab.yp.u.p)[i]
#define  NU  PrmTab.nu.u.i
#define DW(i) ((float*)PrmTab.dw.u.p)[i]
#define  RE  PrmTab.re.u.f
#define  KL  (char*)PrmTab.kl.u.p
#define  HA  PrmTab.ha.u.f
#define  UA  PrmTab.ua.u.f
#define  L0  PrmTab.l0.u.f
#define  A0  PrmTab.a0.u.i
#define  RH  PrmTab.rh.u.i
#define  VD  PrmTab.vd.u.f
#define  VS  PrmTab.vs.u.f
#define  AR  PrmTab.ar.u.f
#define  UR  PrmTab.ur.u.f
#define  EW  PrmTab.ew.u.f
#define  HM  PrmTab.hm.u.f
#define HS(i) ((float*)PrmTab.hs.u.p)[i]
#define PY(i) ((float*)PrmTab.py.u.p)[i]
#define QY(i) ((float*)PrmTab.qy.u.p)[i]
#define PZ(i) ((float*)PrmTab.pz.u.p)[i]
#define QZ(i) ((float*)PrmTab.qz.u.p)[i]
#define XQ(i) ((float*)PrmTab.xq.u.p)[i]
#define YQ(i) ((float*)PrmTab.yq.u.p)[i]
#define HQ(i) ((float*)PrmTab.hq.u.p)[i]
#define EQ(i) ((float*)PrmTab.eq.u.p)[i]
#define AQ(i) ((float*)PrmTab.aq.u.p)[i]
#define BQ(i) ((float*)PrmTab.bq.u.p)[i]
#define RQ(i) ((float*)PrmTab.rq.u.p)[i]
#define UF(i) ((float*)PrmTab.uf.u.p)[i]
#define WF(i) ((float*)PrmTab.wf.u.p)[i]
#define NI(i) ((int*)PrmTab.ni.u.p)[i]
#define LI(i) ((float*)PrmTab.li.u.p)[i]
#define  NX  PrmTab.nx.u.i
#define  NY  PrmTab.ny.u.i
#define  X0  PrmTab.x0.u.f
#define  Y0  PrmTab.y0.u.f
#define  DD  PrmTab.dd.u.f
#define  QT  PrmTab.qt.u.f
#define  RV  PrmTab.rv.u.f
#define  FA  PrmTab.fa.u.f
#define  FO  (char*)PrmTab.fo.u.p
#define  FI  (char*)PrmTab.fi.u.p
#define  ID  (char*)PrmTab.id.u.p
#define  MO  (char*)PrmTab.mo.u.p
#define  SE  (char*)PrmTab.se.u.p
#define  ND  PrmTab.nd.u.i
#define  LM  PrmTab.lm.u.f
#define  Z0  PrmTab.z0.u.f
#define  AM  PrmTab.am.u.f
#define  DM  PrmTab.dm.u.f
#define  N1  PrmTab.n1.u.i
#define  N2  PrmTab.n2.u.i
#define  MS  PrmTab.ms.u.i
#define  FZ  PrmTab.fz.u.f
#define  HW  PrmTab.hw.u.f
