/*================================================================== vsed.c
 *
 * 1999-11-20 UJ
 *
 *=======================================================================*/
#include <stdio.h>
#include <string.h>
#include <math.h>

char  *argv[5];
float Vsed(float, float);
void  Help(void);
float Best, Reynolds;

void main( int argc, char *argv[] ) {
  float vsed, diam, density;
  if (argc < 3) Help();
  diam = density = 0.;
  sscanf(argv[1], "%f", &diam);
  sscanf(argv[2], "%f", &density);
  if (diam <= 0. || density <= 0.) Help();
  vsed = Vsed(diam, density);
  printf("\nBerechnung der Sedimentationsgeschwindigkeit nach VDI 3782 Blatt 1 (neu):\n");
  printf("Durchmesser (in mu): %9.4f\n", diam);
  printf("Dichte (in g/cm3)  : %9.4f\n", density);
  printf("Bestzahl w         : %9.4f\n", Best);
  printf("Reynoldszahl Nre   : %9.4f\n", Reynolds);
  if (Reynolds > 300)
    printf("Achtung: Reynolds-Zahl > 300\n");
  else {
    if (Best < 0.003) vsed = 0;
    printf("vs (in cm/s)       : %9.4f\n", vsed);
  }
  exit(0);
}

/*===================================================================== Help
 */
void Help() {
  printf("Berechnung der Sedimentationsgeschwindigkeit nach VDI 3782 Blatt 1 (neu)\n");
  printf("Aufruf: vsed <Durchmesser (in mu)> <Dichte (in g/cm3)>\n");
  exit(0);  
}


/*==================================================================== Vsed
 */
float Vsed(             /* Berechnung der Sedimentations-Geschwindigkeit */
                        /* nach VDI 3782/1 neu                           */
  float dm,             /* Durchmesser (in mu)                           */
  float rho )           /* Dichte (in g/cm3)                             */
                        /* Return: Vsed (in cm/s)                        */
  {
  float w, logw, re, a, vg;
  static float aa[7] = {
    -0.318657e1, 
     0.992696e0, 
    -0.153193e-2, 
    -0.987059e-3, 
    -0.578878e-3, 
     0.855176e-4,
    -0.327815e-5 };
  int i;
  w = 4.991e-5*dm*dm*dm*rho;
  if (w < 0.24)  re = w/24;
  else {
    logw = log(w);
    a = aa[6];
    for (i=5; i>=0; i--)  a = aa[i] + a*logw;
    re = exp(a);
  } 
  vg = 1462.*re/dm; 
  Best = w;
  Reynolds = re;
  return vg;
}